******************************************************
* Program: ODBC Configure/Login startup program      *
* Written By: Robert Abram                           *
* Date: 02/00                                        *
*                                                    *
* Comments:                                          *
*                                                    *
*                                                    *
******************************************************

  LOCAL lh_form, fResult, ll_formresult

   * Setup Definitions
	#define ODBC_ADD_DSN        1   && Add data source
	
   * Force FoxPro to not show an error message and login dialog if the connection fails.
   	SQLSetProp(0, "displogin", 3)

   * Pop up Odbc config/login form.
	DO FORM odbc_start 
			
   * Get a handle to the form
	lh_form = _Screen.ActiveForm
	
	lh_form.Set_Mark(1, 1)  && Change marks on form to show the user what we are doing.
	DOEVENTS()  && The DOEVENTS are used to allow the form time to update when we change something
			
   *-----------------------------------------------------
   * Start ODBC Process Here
   * Note: We will hold off on the READ EVENTS until
   *       we have completed the ODBC Process.
   *-----------------------------------------------------

   *-----------------------------------------------------
   * First: load libraries. This is just a filler
   *-----------------------------------------------------
   
   * Call any SET LIBRARY TO, etc... here
   
  * Set the first mark to pass
   lh_form.Set_Mark(1, 3)
   DOEVENTS()
   
   *-----------------------------------------------------
   * Second: Test ODBC Connection
   ******************************************************
    
    IF TestConnection(lh_form) = .F.
    	
    	MessageBox("Program was unable to Connect to the SQL Server.  Unable to continue.", 48, "Connection Message")
    	lh_form.bn_cancel.Caption = "\<Close"
    	
      ELSE
      
      	lh_form.Show_Login()
    	
    ENDIF

   * Ok, call the read events so the user can interact with the form.
	READ EVENTS
	
   * Get the login result from the form to see if the user logged in or not.
   * ll_formresult will be .T. if the login button was clicked or .F. if the Cancel/Close button was clicked.
	ll_formresult = lh_form.retval
	
   * Close Form
    RELEASE WINDOW (lh_form.Name)

   * Reset Dialog Login prompt default
   	SQLSetProp(0, "displogin", 1)
   	
   	CANCEL
	
   RETURN
	
		
   *****************************************************
   * PROCEDURE TestConnection
   *****************************************************
   	PROCEDURE TestConnection
   	  LPARAMETERS lh_form
   	  LOCAL sqlhandle 
   	  
   	   lh_form.Set_Mark(2, 1)
   	   DOEVENTS()
   	   
   	   sqlhandle = SQLConnect("testodbc", "xuser", "xpassword")
   	      	      	   
   	   IF (sqlhandle > 0)
   	   
   	   		lh_form.Set_Mark(2, 3)
   	   		DOEVENTS()
   	   
   	   		SQLDisconnect(sqlhandle)
   	   		RETURN .T.
   	   		
   	   	 ELSE
   	   	 
   	   	 	lh_form.Set_Mark(2, 2)
   	   	 	DOEVENTS()
   	   	 	
   	   	 	lh_form.Show_Config()
   	   	 	DOEVENTS()
   	   	 	
   	   	 	IF ConfigureODBC(lh_form) = .F.  && Unable to create connection.
   	   	 	
   	   	 	   * Code here to handle a failed configuration attempt.
   	   	 	  ELSE
   	   	 	  	
   	   	 	  	lh_form.Set_Mark(5, 1)
   	   	 	  	DOEVENTS()
   	   	 	  	
   	   	 	   * Re-Check SQL Connection
   	   	 	    sqlhandle = SQLConnect("testodbc", "xuser", "xpassword")
   	   	 	       	   	 	  	   	   	 	  	  	   
   	   			IF (sqlhandle > 0)
   	   
   	   				SQLDisconnect(sqlhandle)  				
   	   				lh_form.Set_Mark(5, 3)
   	   				DOEVENTS()
   	   				
   	   				RETURN .T.
   	   			ENDIF
   	   	 	  
   	   	 	  	lh_form.Set_Mark(5, 2)
   	   	 	  	DOEVENTS()
 	   	 	  	   	   	 	
   	   	 	ENDIF
   	   ENDIF   	   
   	   
   RETURN .F.
   
   
    
   
   *****************************************************   
   * PROCEDURE ConfigureODBC
   *****************************************************
    PROCEDURE ConfigureODBC
      LPARAMETERS lh_form
    
       LOCAL ln_size, ln_newsize, lc_drivers
    
    	DECLARE Integer SQLGetInstalledDrivers in odbccp32.dll String @, Short, Short @
    		
	  	ln_size = -1
	  	ln_newsize = 0
  		lc_drivers = ""
  		
  		lh_form.Set_Mark(3, 1)
  		DOEVENTS()
	
	   * This first call is to get the size of the buffer we need to hold the list of drivers.
		SQLGetInstalledDrivers(@lc_drivers, ln_size, @ln_newsize)
		
  		IF ln_newsize > 0
  		
  		 * Set size needed to hold the list of drivers
		  lc_drivers = SPACE(ln_newsize)
		  ln_size = ln_newsize

           * Ok, get the list of drivers.
	  		IF SQLGetInstalledDrivers(@lc_drivers, ln_size, @ln_newsize) = 1

               * Check to see if our SQL Server driver is installed.
		      	IF AT("SQL Server", lc_drivers) > 1
		      	
		      		lh_form.Set_Mark(3, 3)
		      		DOEVENTS()
        	
        			RETURN ConfigureODBCConnection(lh_form)
      			ENDIF
    		ENDIF
  		ENDIF   	
    
    	lh_form.Set_Mark(3, 2)
    	DOEVENTS()
    
    RETURN .F.
    
    
    
  
   ****************************************************   
   * PROCEDURE ConfigureODBCConnection
   *****************************************************
	PROCEDURE ConfigureODBCConnection
	  LPARAMETERS lh_form

		DECLARE Integer SQLConfigDataSource IN odbccp32.dll Integer, Short, String @, String @	

		lc_driver = "SQL Server" + CHR(0)
		
		lh_form.Set_Mark(4, 1)
		DOEVENTS()

       * Each part of the configuration string must be delimited with a CHR(0), nothing else
       * NOTE: The DSN used here should be the same as the SQLConnect("xyz_odbc", ...) used above for the final product.
		lc_dsn = "dsn=testodbc" + CHR(0) + ;       
       			 "server=bigserver" + CHR(0) + ;  
       			 "database=bigdb" + CHR(0) + ;    
       			 "network=DBMSSOCN" + CHR(0)  
       			 
		IF SQLConfigDataSource(0, ODBC_ADD_DSN, @lc_driver, @lc_dsn) = 1
		
			lh_form.Set_Mark(4, 3)
			DOEVENTS()
			
			RETURN .T.   && Successfully Configured DSN
		ENDIF
		
		lh_form.Set_Mark(4, 2)
		DOEVENTS()

	RETURN .F.   && failed
