**********************************************************************
* Program....: FindWCol.prg
* Compiler...: Visual FoxPro 06.00.8492.00 for Windows
* Abstract...: Determine what colors are available
**********************************************************************
LOCAL lnHWND, lnDC, lnStatus, lcTxt, lnPalCount, lcBuff, lnGotCount 

*** Declare necessary DLLs
DeclareDLLS()

*** Get the Windows Handle for the Active Window
lnHWND = GetActiveWindow()

*** Now get the device context for the current window
lnDC= GetDC( lnHWND )

*** Check the Palette Status
lnStatus = GetSystemPaletteUse( lnDC )
IF lnStatus # 1
    lcTxt = IIF( lnStatus = 0, 'System Palette has no static colors', ;
                'System Palette is invalid')
    WAIT WINDOW lcTxt NOWAIT
ENDIF

*** Get Number of Entries in the current Palette
lnPalCount =  GetSystemPaletteEntries( lnDC, 0,0, NULL)
*** Return value is RGB value + a flag, whose values are defined in wingdi.h as:
*** #DEFINE PC_RESERVED    1 = Specifies that the low-order 
***         word of the palette entry designates a hardware palette index
*** #DEFINE PC_EXPLICIT    2 = Specifies that the color be placed in an 
***         unused entry in the system palette instead of being matched 
***         to an existing color in the system palette
*** #DEFINE PC_NOCOLLAPSE  4 = Specifies that the logical palette entry 
***         be used for palette animation. Prevents other windows matching
***         to this color because it changes frquently.

*** Set Buffer to appropriate size (3 Chars for RGB value + 1 for the flag)
lcBuff = SPACE( (lnPalCount) * 4 )
lnGotCount = GetSystemPaletteEntries( lnDC, 0, lnPalCount, @lcBuff )

*** Convert entries to viewable form and dump to a file
lcStr = ALLTRIM(STR(lnPalCount)) + " Color Entries in System Palette"
STRTOFILE( lcStr + CHR(13), 'PalCols.txt')
lcStr = ""

*** Now process the string buffer
FOR lnCnt = 1 TO LEN(lcBuff)
    IF lnCnt%4 = 0
        *** End of a color string, this is the flag character
        lcStr = "RGB( " + lcStr  + " ) : F = " + PADL( ASC( SUBSTR(lcBuff, lnCnt, 1) ), 3, "0") + CHR(13)
        STRTOFILE( lcStr, 'PalCols.txt', .T.)
        lcStr = ""
    ELSE
        *** Extract the color numbers as RGB Values
        lcStr = lcStr + IIF(! EMPTY( lcStr ), ", ", "") ;
              + PADL( ASC( SUBSTR(lcBuff, lnCnt, 1) ), 3, "0")
    ENDIF
NEXT

RETURN





CLEAR
lnStart = SECONDS()
FOR lnCnt =  0 TO 16777215
    lnColor = lnCnt
    lnNear = GetNearestColor( lnDC, lnColor )
    IF lnColor = lnNear
        STRTOFILE( NUMTORGB( lnColor ), 'hicols.txt', .T. )
    ENDIF
NEXT
lnEnd = SECONDS()
? STR( (lnEnd - lnStart)/60, 8, 2)
RETURN

*******************************************************************************
*** Windows API Declarations
*******************************************************************************
FUNCTION DeclareDLLS
*******************************************************************************
*** Get Active Window Handle
*** The GetActiveWindow function retrieves the window handle to the active 
*** window attached to the calling thread's message queue. 
DECLARE INTEGER GetActiveWindow IN WIN32API
*******************************************************************************
*** Get Device Context
*** Retrieves a handle to a display device context (DC) for the client area of 
*** a specified window or for the entire screen
DECLARE INTEGER GetDC IN Win32Api ;
				INTEGER nWHnd
*******************************************************************************
*** Check Palette Status
*** #DEFINE SYSPAL_ERROR     0 = System palette has no static colors
*** #DEFINE SYSPAL_STATIC    1 = System palette has static colors 
*** #DEFINE SYSPAL_NOSTATIC  2 = Device context invalid/not support palettes
*** Note: A 'Static Color' is one that will not change when an application 
*** realizes its logical palette.
DECLARE INTEGER GetSystemPaletteUse IN WIN32API ;
                INTEGER nHDC
*******************************************************************************
*** Check Palette Entries
*** Retrieves a range of entries from the system palette.  Parameters:
*** nHDC        : Handle to the device context. 
*** nStartIndex : Index of first entry to retrieve
*** nEntries    : Number of entries to retrieve
*** cBuffer     : Pointer to string buffer for result (Pass by reference!)
DECLARE INTEGER GetSystemPaletteEntries IN WIN32API ;
                INTEGER nHDC, ;
                INTEGER nStartIndex, ;
                INTEGER nEntries, ;
                STRING  cBuffer 
*******************************************************************************
*** Check Color is Available
*** Returns the nearest available color in the palette to the one passed:
DECLARE INTEGER GetNearestColor IN WIN32API ;
                INTEGER nHDC, ;
                INTEGER nColor
*******************************************************************************
RETURN
*** End API Declarations
*******************************************************************************

********************************************************
*** NUMTORGB.PRG
*** Converts a Colour number into RGB components
*** Returns RGB as comma separated string
*** Syntax: <var> = NumToRGB(<expN>)
********************************************************
FUNCTION NumToRGB
LPARAMETERS tnColNum
LOCAL lcredval, lcgrnval, lcbluval, lnLeft
STORE "000" TO lcredval, lcgrnval, lcbluval
*** Get Blue Component
lcBluVal = PADL(INT(tnColNum/65536),3,"0")
*** Get Green Component
lnLeft = MOD(tnColNum, 65536)
IF lnLeft > 256
	lcGrnVal = PADL(INT(lnLeft/256),3,"0")
	lnLeft = MOD(lnLeft, 256)
ENDIF
*** Get Red Component
lcRedVal = PADL(lnLeft,3,"0")
RETURN (lcRedVal + ',' + lcGrnVal + ',' + lcBluVal)
